// Class CMoveGenerate128, Version 6.4, Aug-2017, Perft Optimize
#include "stdafx.h"

#define MOVEGEN_CAPTURE_NOCAPTURE	0
#define MOVEGEN_CAPTURE_WHITEMAN	1
#define MOVEGEN_CAPTURE_WHITEKING	2
#define MOVEGEN_CAPTURE_BLACKMAN	4
#define MOVEGEN_CAPTURE_BLACKKING	8	

#define RAYMASK_DBR (DFLD7 | DFLD12 | DFLD18 | DFLD23 | DFLD29 | DFLD34 | DFLD40)
#define RAYMASK_DBL (DFLD6 | DGHOST10 | DFLD15 | DFLD20 | DFLD24 | DFLD29 | DFLD33 | DFLD38)
#define RAYMASK_DFR (DFLD45 | DGHOST40 | DFLD36 | DFLD31 | DFLD27 | DFLD22 | DFLD18 | DFLD13)
#define RAYMASK_DFL (DFLD11 | DFLD17 | DFLD22 | DFLD28 | DFLD33 |DFLD39 | DFLD44)

#define uint64 unsigned __int64

CMoveGenerate128::CMoveGenerate128()
{
}

CMoveGenerate128::~CMoveGenerate128()
{
}

bool CMoveGenerate128::Duplicate_Move128(PTSTACK pTDepth, PLOOK rlsp)
{
	PLOOK pLook = pTDepth->FirstMove;
	int iXMoves = pTDepth->iXMoves;

	do {

		if (rlsp->bbField[BB_WHITEPIECE] == pLook->bbField[BB_WHITEPIECE] && 
			rlsp->bbField[BB_BLACKPIECE] == pLook->bbField[BB_BLACKPIECE] &&
			rlsp->bbField[BB_KING] == pLook->bbField[BB_KING])
			return (true);

		++pLook;

	} while (--iXMoves > 1);	// Last Move is Duplicate Move

	return (false);
}

int CMoveGenerate128::QWhiteKingCapture128(PTSTACK pTDepth)
{
	DWORD dKingPosition, dRay;
	BITBOARD bbPosition, bbPosition0;
	BITBOARD *pbbField;

	pbbField = pTDepth->pbbField;

	BITBOARD bbEmpty	= pbbField[BB_EMPTY];
	BITBOARD bbKing		= pbbField[BB_WHITEPIECE] & pbbField[BB_KING];
	BITBOARD bbOpponent = pbbField[BB_BLACKPIECE];

	int iXMoves = pTDepth->iXMoves;

	while (bbKing) {

		BITSCANFORWARD64(&dKingPosition, bbKing);

		bbPosition0 = (BITBOARD)1<<dKingPosition;
		bbKing ^= bbPosition0;

		pTDepth->bbFrom = bbPosition0;

		pbbField[BB_EMPTY] = (bbEmpty | bbPosition0);		// Set bit in Empty Field

		bbPosition = (RAYMASK_DFL >> (DFLDI50 - dKingPosition)) &  ~bbEmpty;
		BITSCANREVERSE64(&dRay, bbPosition | DGHOST0);
		bbPosition = (BITBOARD)1 << dRay;

		if (bbOpponent & bbPosition & DBR(bbEmpty))			// DFL Capture
			WhiteKingNextCapture0_128(pTDepth, DFL(bbPosition), bbOpponent ^ bbPosition);

		bbPosition = (RAYMASK_DFR >> (DFLDI50 - dKingPosition)) &  ~bbEmpty;
		BITSCANREVERSE64(&dRay, bbPosition | DGHOST0);
		bbPosition = (BITBOARD)1 << dRay;

		if (bbOpponent & bbPosition & DBL(bbEmpty))			// DFR Capture					
			WhiteKingNextCapture1_128(pTDepth, DFR(bbPosition), bbOpponent ^ bbPosition);

		bbPosition = (RAYMASK_DBR << (dKingPosition-1)) &  ~bbEmpty;
		bbPosition &= ~bbPosition + 1;						// Least significant Bit

		if (bbOpponent & bbPosition & DFL(bbEmpty))			// DBR Capture
			WhiteKingNextCapture2_128(pTDepth, DBR(bbPosition), bbOpponent ^ bbPosition);

		bbPosition = (RAYMASK_DBL << (dKingPosition-1)) &  ~bbEmpty;
		bbPosition &= ~bbPosition + 1;						// Least significant Bit

		if (bbOpponent & bbPosition & DFR(bbEmpty))			// DBL Capture
			WhiteKingNextCapture3_128(pTDepth, DBL(bbPosition), bbOpponent ^ bbPosition);

		pbbField[BB_EMPTY] = bbEmpty;						// Reset bit in Empty Field
	}

	if (iXMoves == pTDepth->iXMoves) return (MOVEGEN_CAPTURE_NOCAPTURE);

	return (MOVEGEN_CAPTURE_WHITEKING);
}

int CMoveGenerate128::QBlackKingCapture128(PTSTACK pTDepth)
{
	DWORD dKingPosition, dRay;
	BITBOARD bbPosition, bbPosition0;
	BITBOARD *pbbField;

	pbbField = pTDepth->pbbField;

	BITBOARD bbEmpty	= pbbField[BB_EMPTY];
	BITBOARD bbKing		= pbbField[BB_BLACKPIECE] & pbbField[BB_KING];
	BITBOARD bbOpponent = pbbField[BB_WHITEPIECE];

	int iXMoves = pTDepth->iXMoves;

	while (bbKing) {

		BITSCANFORWARD64(&dKingPosition, bbKing);

		bbPosition0 = (BITBOARD)1 << dKingPosition;
		bbKing ^= bbPosition0;

		pTDepth->bbFrom = bbPosition0;

		pbbField[BB_EMPTY] = (bbEmpty | bbPosition0);		// Set bit in Empty Field

		bbPosition = (RAYMASK_DFL >> (DFLDI50 - dKingPosition)) &  ~bbEmpty;
		BITSCANREVERSE64(&dRay, bbPosition | DGHOST0);
		bbPosition = (BITBOARD)1 << dRay;

		if (bbOpponent & bbPosition & DBR(bbEmpty))			// DFL Capture
			BlackKingNextCapture0_128(pTDepth, DFL(bbPosition), bbOpponent ^ bbPosition);

		bbPosition = (RAYMASK_DFR >> (DFLDI50 - dKingPosition)) &  ~bbEmpty;
		BITSCANREVERSE64(&dRay, bbPosition | DGHOST0);
		bbPosition = (BITBOARD)1 << dRay;

		if (bbOpponent & bbPosition & DBL(bbEmpty))			// DFR Capture					
			BlackKingNextCapture1_128(pTDepth, DFR(bbPosition), bbOpponent ^ bbPosition);

		bbPosition = (RAYMASK_DBR << (dKingPosition - 1)) &  ~bbEmpty;
		bbPosition &= ~bbPosition + 1;						// Least significant Bit

		if (bbOpponent & bbPosition & DFL(bbEmpty))			// DBR Capture
			BlackKingNextCapture2_128(pTDepth, DBR(bbPosition), bbOpponent ^ bbPosition);

		bbPosition = (RAYMASK_DBL << (dKingPosition - 1)) &  ~bbEmpty;
		bbPosition &= ~bbPosition + 1;						// Least significant Bit

		if (bbOpponent & bbPosition & DFR(bbEmpty))			// DBL Capture
			BlackKingNextCapture3_128(pTDepth, DBL(bbPosition), bbOpponent ^ bbPosition);

		pbbField[BB_EMPTY] = bbEmpty;						// Reset bit in Empty Field
	}

	if (iXMoves == pTDepth->iXMoves) return (MOVEGEN_CAPTURE_NOCAPTURE);

	return (MOVEGEN_CAPTURE_BLACKKING);
}

int CMoveGenerate128::QWhiteManCapture128(PTSTACK pTDepth)
{
	BITBOARD* pbbField = pTDepth->pbbField;

	BITBOARD bbWhiteMan = pbbField[BB_WHITEPIECE] & ~pbbField[BB_KING];

	if (bbWhiteMan == 0) return (MOVEGEN_CAPTURE_NOCAPTURE);

	BITBOARD bbEmpty = pbbField[BB_EMPTY];

	// WhiteMan Capture 
	BITBOARD bbOpponent = pbbField[BB_BLACKPIECE];

	// Init Capture BitBoards
	BITBOARD bbCapture1 = DBR(bbEmpty) & bbOpponent;
	BITBOARD bbCapture2 = DBL(bbEmpty) & bbOpponent;
	BITBOARD bbCapture3 = DFL(bbEmpty) & bbOpponent;
	BITBOARD bbCapture4 = DFR(bbEmpty) & bbOpponent;

	BITBOARD bbCapture = (bbWhiteMan) & (DBR(bbCapture1) | DBL(bbCapture2) | DFL(bbCapture3) | DFR(bbCapture4));

	if (bbCapture == 0) return (MOVEGEN_CAPTURE_NOCAPTURE);

	while (bbCapture) {

		BITBOARD bbPosition = bbCapture & (~bbCapture + 1);
		bbCapture ^= bbPosition;

		pTDepth->bbFrom = bbPosition;

		pbbField[BB_EMPTY] |= bbPosition;	// Set bit in Empty Field

		if (DFR(bbEmpty) & bbOpponent & DBL(bbPosition))
			WhiteManNextCapture128(pTDepth, DBLBL(bbPosition), bbOpponent ^ DBL(bbPosition));

		if (DFL(bbEmpty) & bbOpponent & DBR(bbPosition))
			WhiteManNextCapture128(pTDepth, DBRBR(bbPosition), bbOpponent ^ DBR(bbPosition));

		if (DBR(bbEmpty) & bbOpponent & DFL(bbPosition))
			WhiteManNextCapture128(pTDepth, DFLFL(bbPosition), bbOpponent ^ DFL(bbPosition));

		if (DBL(bbEmpty) & bbOpponent & DFR(bbPosition))
			WhiteManNextCapture128(pTDepth, DFRFR(bbPosition), bbOpponent ^ DFR(bbPosition));

		pbbField[BB_EMPTY] ^= bbPosition;	// Reset bit in Empty Field
	}

	return(MOVEGEN_CAPTURE_WHITEMAN);
}

int CMoveGenerate128::QBlackManCapture128(PTSTACK pTDepth)
{
	BITBOARD* pbbField = pTDepth->pbbField;

	BITBOARD bbBlackMan = pbbField[BB_BLACKPIECE] & ~pbbField[BB_KING];

	if (bbBlackMan == 0) return (MOVEGEN_CAPTURE_NOCAPTURE);

	BITBOARD bbEmpty = pbbField[BB_EMPTY];

	// BlackMan Capture
	BITBOARD bbOpponent = pbbField[BB_WHITEPIECE];

	// Init Capture BitBoards
	BITBOARD bbCapture1 = DBR(bbEmpty) & bbOpponent;
	BITBOARD bbCapture2 = DBL(bbEmpty) & bbOpponent;
	BITBOARD bbCapture3 = DFL(bbEmpty) & bbOpponent;
	BITBOARD bbCapture4 = DFR(bbEmpty) & bbOpponent;

	BITBOARD bbCapture = (bbBlackMan) & (DBR(bbCapture1) | DBL(bbCapture2) | DFL(bbCapture3) | DFR(bbCapture4));

	if (bbCapture == 0 ) return (MOVEGEN_CAPTURE_NOCAPTURE);

	while (bbCapture) {

		BITBOARD bbPosition = bbCapture & (~bbCapture + 1);
		bbCapture ^= bbPosition;

		pTDepth->bbFrom = bbPosition;

		pbbField[BB_EMPTY] |= bbPosition;	// Set bit in Empty Field

		if (DFR(bbEmpty) & bbOpponent & DBL(bbPosition))
			BlackManNextCapture128(pTDepth, DBLBL(bbPosition), bbOpponent ^ DBL(bbPosition));

		if (DFL(bbEmpty) & bbOpponent & DBR(bbPosition))
			BlackManNextCapture128(pTDepth, DBRBR(bbPosition), bbOpponent ^ DBR(bbPosition));

		if (DBR(bbEmpty) & bbOpponent & DFL(bbPosition))
			BlackManNextCapture128(pTDepth, DFLFL(bbPosition), bbOpponent ^ DFL(bbPosition));

		if (DBL(bbEmpty) & bbOpponent & DFR(bbPosition))
			BlackManNextCapture128(pTDepth, DFRFR(bbPosition), bbOpponent ^ DFR(bbPosition));

		pbbField[BB_EMPTY] ^= bbPosition;	// Reset bit in Empty Field
	}

	return(MOVEGEN_CAPTURE_BLACKMAN);
}

void CMoveGenerate128::WhiteManNextCapture128(PTSTACK pTDepth, BITBOARD bbPosition, BITBOARD bbOpponent)
{
	PLOOK rlsp;

	BITBOARD* pbbField = pTDepth->pbbField;
	BITBOARD bbEmpty = pbbField[BB_EMPTY];

	if (DBLBL(bbPosition) & DBL(bbOpponent) & bbEmpty)
		WhiteManNextCapture128(pTDepth, DBLBL(bbPosition), bbOpponent ^ (DBL(bbPosition)));

	if (DBRBR(bbPosition) & DBR(bbOpponent) & bbEmpty)
		WhiteManNextCapture128(pTDepth, DBRBR(bbPosition), bbOpponent ^ (DBR(bbPosition)));

	if (DFLFL(bbPosition) & DFL(bbOpponent) & bbEmpty)
		WhiteManNextCapture128(pTDepth, DFLFL(bbPosition), bbOpponent ^ (DFL(bbPosition)));

	if (DFRFR(bbPosition) & DFR(bbOpponent) & bbEmpty)
		WhiteManNextCapture128(pTDepth, DFRFR(bbPosition), bbOpponent ^ (DFR(bbPosition)));

	BITBOARD bbCaptured = pbbField[BB_BLACKPIECE] ^ bbOpponent;
	int iXCapture = (int)POPCNT64(bbCaptured);

	if (iXCapture >= pTDepth->iXCapture) {

		if (iXCapture == pTDepth->iXCapture) {
			++pTDepth->iXMoves;
			rlsp = pTDepth->NextMove;
		} else {
			pTDepth->iXMoves = 1;
			pTDepth->iXCapture = iXCapture;
			rlsp = pTDepth->FirstMove;
		}

		rlsp->bbField[BB_WHITEPIECE]	= (pbbField[BB_WHITEPIECE] ^ (pTDepth->bbFrom ^ bbPosition));
		rlsp->bbField[BB_BLACKPIECE]	= bbOpponent;
		rlsp->bbField[BB_KING]			= (pbbField[BB_KING] & ~bbCaptured) | (bbPosition & DFLD_PROMOTION_WHITE);

		if (pTDepth->iXCapture < 4 || pTDepth->iXMoves == 1 || !Duplicate_Move128(pTDepth, rlsp))
			pTDepth->NextMove = rlsp + 1;
		else
			--pTDepth->iXMoves;
	}
}

void CMoveGenerate128::BlackManNextCapture128(PTSTACK pTDepth, BITBOARD bbPosition, BITBOARD bbOpponent)
{
	PLOOK rlsp;

	BITBOARD* pbbField = pTDepth->pbbField;
	BITBOARD bbEmpty = pbbField[BB_EMPTY];

	if (DBLBL(bbPosition) & DBL(bbOpponent) & bbEmpty)
		BlackManNextCapture128(pTDepth, DBLBL(bbPosition), bbOpponent ^ (DBL(bbPosition)));

	if (DBRBR(bbPosition) & DBR(bbOpponent) & bbEmpty)
		BlackManNextCapture128(pTDepth, DBRBR(bbPosition), bbOpponent ^ (DBR(bbPosition)));

	if (DFLFL(bbPosition) & DFL(bbOpponent) & bbEmpty)
		BlackManNextCapture128(pTDepth, DFLFL(bbPosition), bbOpponent ^ (DFL(bbPosition)));

	if (DFRFR(bbPosition) & DFR(bbOpponent) & bbEmpty)
		BlackManNextCapture128(pTDepth, DFRFR(bbPosition), bbOpponent ^ (DFR(bbPosition)));

	BITBOARD bbCaptured = pbbField[BB_WHITEPIECE] ^ bbOpponent;
	int iXCapture = (int)POPCNT64(bbCaptured);

	if (iXCapture >= pTDepth->iXCapture) {

		if (iXCapture == pTDepth->iXCapture) {
			++pTDepth->iXMoves;
			rlsp = pTDepth->NextMove;
		} else {
			pTDepth->iXMoves = 1;
			pTDepth->iXCapture = iXCapture;
			rlsp = pTDepth->FirstMove;
		}

		rlsp->bbField[BB_WHITEPIECE]	= bbOpponent;
		rlsp->bbField[BB_BLACKPIECE]	= (pbbField[BB_BLACKPIECE] ^ (pTDepth->bbFrom ^ bbPosition));
		rlsp->bbField[BB_KING]			= (pbbField[BB_KING] & ~bbCaptured) | (bbPosition & DFLD_PROMOTION_BLACK);

		if (pTDepth->iXCapture < 4 || pTDepth->iXMoves == 1 || !Duplicate_Move128(pTDepth, rlsp))
			pTDepth->NextMove = rlsp + 1;
		else
			--pTDepth->iXMoves;
	}
}

//	KingNextCapture0()
//	Direction = DFL, so Left = DBL, Right = DFR, Forward = DFL
//
void CMoveGenerate128::WhiteKingNextCapture0_128(PTSTACK pTDepth, BITBOARD bbPosition, BITBOARD bbOpponent)
{
	DWORD dKingPosition, dRay;
	PLOOK rlsp;
	BITBOARD bbEmpty, bbPositionNext, *pbbField;

	pbbField = pTDepth->pbbField;
	bbEmpty = pbbField[BB_EMPTY];

	while (true) {

		BITSCANFORWARD64(&dKingPosition, bbPosition);

		bbPositionNext = (RAYMASK_DBL << (dKingPosition - 1)) &  ~bbEmpty;			// Left Direction
		bbPositionNext &= ~bbPositionNext + 1;

		if (bbOpponent & bbPositionNext & DFR(bbEmpty))
			WhiteKingNextCapture3_128(pTDepth, DBL(bbPositionNext), bbOpponent ^ bbPositionNext);

		bbPositionNext = (RAYMASK_DFR >> (DFLDI50 - dKingPosition)) &  ~bbEmpty;	// Right Direction
		BITSCANREVERSE64(&dRay, bbPositionNext | DGHOST0);
		bbPositionNext = (BITBOARD)1 << dRay;

		if (bbOpponent & bbPositionNext & DBL(bbEmpty))
			WhiteKingNextCapture1_128(pTDepth, DFR(bbPositionNext), bbOpponent ^ bbPositionNext);

		BITBOARD bbCaptured = pbbField[BB_BLACKPIECE] ^ bbOpponent;
		int iXCapture = (int)POPCNT64(bbCaptured);

		if (iXCapture >= pTDepth->iXCapture) {										// Update Capture-array 

			if (iXCapture == pTDepth->iXCapture) {
				++pTDepth->iXMoves;
				rlsp = pTDepth->NextMove;
			} else {
				pTDepth->iXMoves = 1;
				pTDepth->iXCapture = iXCapture;
				rlsp = pTDepth->FirstMove;
			}

			rlsp->bbField[BB_WHITEPIECE]	= (pbbField[BB_WHITEPIECE] ^ (pTDepth->bbFrom ^ bbPosition));
			rlsp->bbField[BB_BLACKPIECE]	= bbOpponent;
			rlsp->bbField[BB_KING]			= (pbbField[BB_KING] & ~bbCaptured) ^ (pTDepth->bbFrom ^ bbPosition);

			if (pTDepth->iXCapture < 4 || pTDepth->iXMoves == 1 || !Duplicate_Move128(pTDepth, rlsp))
				pTDepth->NextMove = rlsp + 1;
			else
				--pTDepth->iXMoves;
		}

		if (!(bbEmpty & (bbPosition = DFL(bbPosition)))) {

			if (bbOpponent & bbPosition & DBR(bbEmpty))
				WhiteKingNextCapture0_128(pTDepth, DFL(bbPosition), bbOpponent ^ bbPosition);

			break;
		}
	}
}

//	KingNextCapture1()
//	Direction = DFR, so Left = DFL, Right = DBR, Forward = DFR
//
void CMoveGenerate128::WhiteKingNextCapture1_128(PTSTACK pTDepth, BITBOARD bbPosition, BITBOARD bbOpponent)
{
	DWORD dKingPosition, dRay;
	PLOOK rlsp;
	BITBOARD bbPositionNext;

	BITBOARD* pbbField = pTDepth->pbbField;
	BITBOARD bbEmpty = pbbField[BB_EMPTY];

	while (true) {

		BITSCANFORWARD64(&dKingPosition, bbPosition);

		bbPositionNext = (RAYMASK_DFL >> (DFLDI50 - dKingPosition)) &  ~bbEmpty;	// Left Direction
		BITSCANREVERSE64(&dRay, bbPositionNext | DGHOST0);
		bbPositionNext = (BITBOARD)1 << dRay;

		if (bbOpponent & bbPositionNext & DBR(bbEmpty))
			WhiteKingNextCapture0_128(pTDepth, DFL(bbPositionNext), bbOpponent ^ bbPositionNext);

		BITSCANFORWARD64(&dKingPosition, bbPosition);
		bbPositionNext = (RAYMASK_DBR << (dKingPosition - 1)) &  ~bbEmpty;			// Right Direction
		bbPositionNext &= ~bbPositionNext + 1;

		if (bbOpponent & bbPositionNext & DFL(bbEmpty))
			WhiteKingNextCapture2_128(pTDepth, DBR(bbPositionNext), bbOpponent ^ bbPositionNext);

		BITBOARD bbCaptured = pbbField[BB_BLACKPIECE] ^ bbOpponent;
		int iXCapture = (int)POPCNT64(bbCaptured);

		if (iXCapture >= pTDepth->iXCapture) {										// Update Capture-array 

			if (iXCapture == pTDepth->iXCapture) {
				++pTDepth->iXMoves;
				rlsp = pTDepth->NextMove;
			}
			else {
				pTDepth->iXMoves = 1;
				pTDepth->iXCapture = iXCapture;
				rlsp = pTDepth->FirstMove;
			}

			rlsp->bbField[BB_WHITEPIECE] = (pbbField[BB_WHITEPIECE] ^ (pTDepth->bbFrom ^ bbPosition));
			rlsp->bbField[BB_BLACKPIECE] = bbOpponent;
			rlsp->bbField[BB_KING] = (pbbField[BB_KING] & ~bbCaptured) ^ (pTDepth->bbFrom ^ bbPosition);
	
			if (pTDepth->iXCapture < 4 || pTDepth->iXMoves == 1 || !Duplicate_Move128(pTDepth, rlsp))
				pTDepth->NextMove = rlsp + 1;
			else
				--pTDepth->iXMoves;
		}

		if (!(bbEmpty & (bbPosition = DFR(bbPosition)))) {

			if (bbOpponent & bbPosition & DBL(bbEmpty))
				WhiteKingNextCapture1_128(pTDepth, DFR(bbPosition), bbOpponent ^ bbPosition);

			break;
		}
	}
}

//	KingNextCapture2()
//	Direction = DBR, so Left = DFR, Right = DBL, Forward = DBR
//
void CMoveGenerate128::WhiteKingNextCapture2_128(PTSTACK pTDepth, BITBOARD bbPosition, BITBOARD bbOpponent)
{
	DWORD dKingPosition, dRay;
	PLOOK rlsp;
	BITBOARD bbPositionNext;

	BITBOARD* pbbField = pTDepth->pbbField;
	BITBOARD bbEmpty = pbbField[BB_EMPTY];

	while (true) {

		BITSCANFORWARD64(&dKingPosition, bbPosition);

		bbPositionNext = (RAYMASK_DFR >> (DFLDI50 - dKingPosition)) &  ~bbEmpty;	// Left Direction
		BITSCANREVERSE64(&dRay, bbPositionNext | DGHOST0);
		bbPositionNext = (BITBOARD)1 << dRay;

		if (bbOpponent & bbPositionNext & DBL(bbEmpty))
			WhiteKingNextCapture1_128(pTDepth, DFR(bbPositionNext), bbOpponent ^ bbPositionNext);

		bbPositionNext = (RAYMASK_DBL << (dKingPosition - 1)) &  ~bbEmpty;			// Right Direction
		bbPositionNext &= ~bbPositionNext + 1;

		if (bbOpponent & bbPositionNext & DFR(bbEmpty))
			WhiteKingNextCapture3_128(pTDepth, DBL(bbPositionNext), bbOpponent ^ bbPositionNext);

		BITBOARD bbCaptured = pbbField[BB_BLACKPIECE] ^ bbOpponent;
		int iXCapture = (int)POPCNT64(bbCaptured);

		if (iXCapture >= pTDepth->iXCapture) {										// Update Capture-array 

			if (iXCapture == pTDepth->iXCapture) {
				++pTDepth->iXMoves;
				rlsp = pTDepth->NextMove;
			} else {
				pTDepth->iXMoves = 1;
				pTDepth->iXCapture = iXCapture;
				rlsp = pTDepth->FirstMove;
			}

			rlsp->bbField[BB_WHITEPIECE] = (pbbField[BB_WHITEPIECE] ^ (pTDepth->bbFrom ^ bbPosition));
			rlsp->bbField[BB_BLACKPIECE] = bbOpponent;
			rlsp->bbField[BB_KING] = (pbbField[BB_KING] & ~bbCaptured) ^ (pTDepth->bbFrom ^ bbPosition);

			if (pTDepth->iXCapture < 4 || pTDepth->iXMoves == 1 || !Duplicate_Move128(pTDepth, rlsp))
				pTDepth->NextMove = rlsp + 1;
			else
				--pTDepth->iXMoves;
		}

		if (!(bbEmpty & (bbPosition = DBR(bbPosition)))) {

			if (bbOpponent & bbPosition & DFL(bbEmpty))
				WhiteKingNextCapture2_128(pTDepth, DBR(bbPosition), bbOpponent ^ bbPosition);

			break;
		}
	}
}

//	KingNextCapture3()
//	Direction = DBL, so Left = DBR, Right = DFL, Forward = DBL
//
void CMoveGenerate128::WhiteKingNextCapture3_128(PTSTACK pTDepth, BITBOARD bbPosition, BITBOARD bbOpponent)
{
	DWORD dKingPosition, dRay;
	PLOOK rlsp;
	BITBOARD bbPositionNext;

	BITBOARD* pbbField = pTDepth->pbbField;
	BITBOARD bbEmpty = pbbField[BB_EMPTY];

	while (true) {

		BITSCANFORWARD64(&dKingPosition, bbPosition);

		bbPositionNext = (RAYMASK_DBR << (dKingPosition - 1)) &  ~bbEmpty;			// Left Direction
		bbPositionNext &= ~bbPositionNext + 1;

		if (bbOpponent & bbPositionNext & DFL(bbEmpty))
			WhiteKingNextCapture2_128(pTDepth, DBR(bbPositionNext), bbOpponent ^ bbPositionNext);

		bbPositionNext = (RAYMASK_DFL >> (DFLDI50 - dKingPosition)) &  ~bbEmpty;	// Right Direction
		BITSCANREVERSE64(&dRay, bbPositionNext | DGHOST0);
		bbPositionNext = (BITBOARD)1 << dRay;

		if (bbOpponent & bbPositionNext & DBR(bbEmpty))
			WhiteKingNextCapture0_128(pTDepth, DFL(bbPositionNext), bbOpponent ^ bbPositionNext);

		BITBOARD bbCaptured = pbbField[BB_BLACKPIECE] ^ bbOpponent;
		int iXCapture = (int)POPCNT64(bbCaptured);

		if (iXCapture >= pTDepth->iXCapture) {										// Update Capture-array 

			if (iXCapture == pTDepth->iXCapture) {
				++pTDepth->iXMoves;
				rlsp = pTDepth->NextMove;
			}
			else {
				pTDepth->iXMoves = 1;
				pTDepth->iXCapture = iXCapture;
				rlsp = pTDepth->FirstMove;
			}

			rlsp->bbField[BB_WHITEPIECE] = (pbbField[BB_WHITEPIECE] ^ (pTDepth->bbFrom ^ bbPosition));
			rlsp->bbField[BB_BLACKPIECE] = bbOpponent;
			rlsp->bbField[BB_KING] = (pbbField[BB_KING] & ~bbCaptured) ^ (pTDepth->bbFrom ^ bbPosition);

			if (pTDepth->iXCapture < 4 || pTDepth->iXMoves == 1 || !Duplicate_Move128(pTDepth, rlsp))
				pTDepth->NextMove = rlsp + 1;
			else
				--pTDepth->iXMoves;
		}

		if (!(bbEmpty & (bbPosition = DBL(bbPosition)))) {

			if (bbOpponent & bbPosition & DFR(bbEmpty))
				WhiteKingNextCapture3_128(pTDepth, DBL(bbPosition), bbOpponent ^ bbPosition);

			break;
		}
	}
}
//	KingNextCapture0()
//	Direction = DFL, so Left = DBL, Right = DFR, Forward = DFL
//
void CMoveGenerate128::BlackKingNextCapture0_128(PTSTACK pTDepth, BITBOARD bbPosition, BITBOARD bbOpponent)
{
	DWORD dKingPosition, dRay;
	PLOOK rlsp;
	BITBOARD bbPositionNext;

	BITBOARD* pbbField = pTDepth->pbbField;
	BITBOARD bbEmpty = pbbField[BB_EMPTY];

	while (true) {

		BITSCANFORWARD64(&dKingPosition, bbPosition);

		bbPositionNext = (RAYMASK_DBL << (dKingPosition - 1)) &  ~bbEmpty;			// Left Direction
		bbPositionNext &= ~bbPositionNext + 1;

		if (bbOpponent & bbPositionNext & DFR(bbEmpty))
			BlackKingNextCapture3_128(pTDepth, DBL(bbPositionNext), bbOpponent ^ bbPositionNext);

		bbPositionNext = (RAYMASK_DFR >> (DFLDI50 - dKingPosition)) &  ~bbEmpty;	// Right Direction
		BITSCANREVERSE64(&dRay, bbPositionNext | DGHOST0);
		bbPositionNext = (BITBOARD)1 << dRay;

		if (bbOpponent & bbPositionNext & DBL(bbEmpty))
			BlackKingNextCapture1_128(pTDepth, DFR(bbPositionNext), bbOpponent ^ bbPositionNext);

		BITBOARD bbCaptured = pbbField[BB_WHITEPIECE] ^ bbOpponent;
		int iXCapture = (int)POPCNT64(bbCaptured);

		if (iXCapture >= pTDepth->iXCapture) {										// Update Capture-array 

			if (iXCapture == pTDepth->iXCapture) {
				++pTDepth->iXMoves;
				rlsp = pTDepth->NextMove;
			}
			else {
				pTDepth->iXMoves = 1;
				pTDepth->iXCapture = iXCapture;
				rlsp = pTDepth->FirstMove;
			}

			rlsp->bbField[BB_WHITEPIECE] = bbOpponent;
			rlsp->bbField[BB_BLACKPIECE] = (pbbField[BB_BLACKPIECE] ^ (pTDepth->bbFrom ^ bbPosition));
			rlsp->bbField[BB_KING] = (pbbField[BB_KING] & ~bbCaptured) ^ (pTDepth->bbFrom ^ bbPosition);

			if (pTDepth->iXCapture < 4 || pTDepth->iXMoves == 1 || !Duplicate_Move128(pTDepth, rlsp))
				pTDepth->NextMove = rlsp + 1;
			else
				--pTDepth->iXMoves;
		}

		if (!(bbEmpty & (bbPosition = DFL(bbPosition)))) {

			if (bbOpponent & bbPosition & DBR(bbEmpty))
				BlackKingNextCapture0_128(pTDepth, DFL(bbPosition), bbOpponent ^ bbPosition);

			break;
		}
	}
}

//	KingNextCapture1()
//	Direction = DFR, so Left = DFL, Right = DBR, Forward = DFR
//
void CMoveGenerate128::BlackKingNextCapture1_128(PTSTACK pTDepth, BITBOARD bbPosition, BITBOARD bbOpponent)
{
	DWORD dKingPosition, dRay;
	PLOOK rlsp;
	BITBOARD bbPositionNext;

	BITBOARD* pbbField = pTDepth->pbbField;
	BITBOARD bbEmpty = pbbField[BB_EMPTY];

	while (true) {

		BITSCANFORWARD64(&dKingPosition, bbPosition);

		bbPositionNext = (RAYMASK_DFL >> (DFLDI50 - dKingPosition)) &  ~bbEmpty;	// Left Direction
		BITSCANREVERSE64(&dRay, bbPositionNext | DGHOST0);
		bbPositionNext = (BITBOARD)1 << dRay;

		if (bbOpponent & bbPositionNext & DBR(bbEmpty))
			BlackKingNextCapture0_128(pTDepth, DFL(bbPositionNext), bbOpponent ^ bbPositionNext);

		BITSCANFORWARD64(&dKingPosition, bbPosition);
		bbPositionNext = (RAYMASK_DBR << (dKingPosition - 1)) &  ~bbEmpty;			// Right Direction
		bbPositionNext &= ~bbPositionNext + 1;

		if (bbOpponent & bbPositionNext & DFL(bbEmpty))
			BlackKingNextCapture2_128(pTDepth, DBR(bbPositionNext), bbOpponent ^ bbPositionNext);

		BITBOARD bbCaptured = pbbField[BB_WHITEPIECE] ^ bbOpponent;
		int iXCapture = (int)POPCNT64(bbCaptured);

		if (iXCapture >= pTDepth->iXCapture) {										// Update Capture-array 

			if (iXCapture == pTDepth->iXCapture) {
				++pTDepth->iXMoves;
				rlsp = pTDepth->NextMove;
			}
			else {
				pTDepth->iXMoves = 1;
				pTDepth->iXCapture = iXCapture;
				rlsp = pTDepth->FirstMove;
			}

			rlsp->bbField[BB_WHITEPIECE] = bbOpponent;
			rlsp->bbField[BB_BLACKPIECE] = (pbbField[BB_BLACKPIECE] ^ (pTDepth->bbFrom ^ bbPosition));
			rlsp->bbField[BB_KING] = (pbbField[BB_KING] & ~bbCaptured) ^ (pTDepth->bbFrom ^ bbPosition);

			if (pTDepth->iXCapture < 4 || pTDepth->iXMoves == 1 || !Duplicate_Move128(pTDepth, rlsp))
				pTDepth->NextMove = rlsp + 1;
			else
				--pTDepth->iXMoves;
		}

		if (!(bbEmpty & (bbPosition = DFR(bbPosition)))) {

			if (bbOpponent & bbPosition & DBL(bbEmpty))
				BlackKingNextCapture1_128(pTDepth, DFR(bbPosition), bbOpponent ^ bbPosition);

			break;
		}
	}
}

//	KingNextCapture2()
//	Direction = DBR, so Left = DFR, Right = DBL, Forward = DBR
//
void CMoveGenerate128::BlackKingNextCapture2_128(PTSTACK pTDepth, BITBOARD bbPosition, BITBOARD bbOpponent)
{
	DWORD dKingPosition, dRay;
	PLOOK rlsp;
	BITBOARD bbPositionNext;

	BITBOARD* pbbField = pTDepth->pbbField;
	BITBOARD bbEmpty = pbbField[BB_EMPTY];

	while (true) {

		BITSCANFORWARD64(&dKingPosition, bbPosition);

		bbPositionNext = (RAYMASK_DFR >> (DFLDI50 - dKingPosition)) &  ~bbEmpty;	// Left Direction
		BITSCANREVERSE64(&dRay, bbPositionNext | DGHOST0);
		bbPositionNext = (BITBOARD)1 << dRay;

		if (bbOpponent & bbPositionNext & DBL(bbEmpty))
			BlackKingNextCapture1_128(pTDepth, DFR(bbPositionNext), bbOpponent ^ bbPositionNext);

		bbPositionNext = (RAYMASK_DBL << (dKingPosition - 1)) &  ~bbEmpty;			// Right Direction
		bbPositionNext &= ~bbPositionNext + 1;

		if (bbOpponent & bbPositionNext & DFR(bbEmpty))
			BlackKingNextCapture3_128(pTDepth, DBL(bbPositionNext), bbOpponent ^ bbPositionNext);

		BITBOARD bbCaptured = pbbField[BB_WHITEPIECE] ^ bbOpponent;
		int iXCapture = (int)POPCNT64(bbCaptured);

		if (iXCapture >= pTDepth->iXCapture) {										// Update Capture-array 

			if (iXCapture == pTDepth->iXCapture) {
				++pTDepth->iXMoves;
				rlsp = pTDepth->NextMove;
			}
			else {
				pTDepth->iXMoves = 1;
				pTDepth->iXCapture = iXCapture;
				rlsp = pTDepth->FirstMove;
			}

			rlsp->bbField[BB_WHITEPIECE] = bbOpponent;
			rlsp->bbField[BB_BLACKPIECE] = (pbbField[BB_BLACKPIECE] ^ (pTDepth->bbFrom ^ bbPosition));
			rlsp->bbField[BB_KING] = (pbbField[BB_KING] & ~bbCaptured) ^ (pTDepth->bbFrom ^ bbPosition);

			if (pTDepth->iXCapture < 4 || pTDepth->iXMoves == 1 || !Duplicate_Move128(pTDepth, rlsp))
				pTDepth->NextMove = rlsp + 1;
			else
				--pTDepth->iXMoves;
		}

		if (!(bbEmpty & (bbPosition = DBR(bbPosition)))) {

			if (bbOpponent & bbPosition & DFL(bbEmpty))
				BlackKingNextCapture2_128(pTDepth, DBR(bbPosition), bbOpponent ^ bbPosition);

			break;
		}
	}
}

//	KingNextCapture3()
//	Direction = DBL, so Left = DBR, Right = DFL, Forward = DBL
//
void CMoveGenerate128::BlackKingNextCapture3_128(PTSTACK pTDepth, BITBOARD bbPosition, BITBOARD bbOpponent)
{
	DWORD dKingPosition, dRay;
	PLOOK rlsp;
	BITBOARD bbPositionNext;

	BITBOARD* pbbField = pTDepth->pbbField;
	BITBOARD bbEmpty = pbbField[BB_EMPTY];

	while (true) {

		BITSCANFORWARD64(&dKingPosition, bbPosition);

		bbPositionNext = (RAYMASK_DBR << (dKingPosition - 1)) &  ~bbEmpty;			// Left Direction
		bbPositionNext &= ~bbPositionNext + 1;

		if (bbOpponent & bbPositionNext & DFL(bbEmpty))
			BlackKingNextCapture2_128(pTDepth, DBR(bbPositionNext), bbOpponent ^ bbPositionNext);

		bbPositionNext = (RAYMASK_DFL >> (DFLDI50 - dKingPosition)) &  ~bbEmpty;	// Right Direction
		BITSCANREVERSE64(&dRay, bbPositionNext | DGHOST0);
		bbPositionNext = (BITBOARD)1 << dRay;

		if (bbOpponent & bbPositionNext & DBR(bbEmpty))
			BlackKingNextCapture0_128(pTDepth, DFL(bbPositionNext), bbOpponent ^ bbPositionNext);

		BITBOARD bbCaptured = pbbField[BB_WHITEPIECE] ^ bbOpponent;
		int iXCapture = (int)POPCNT64(bbCaptured);

		if (iXCapture >= pTDepth->iXCapture) {										// Update Capture-array 

			if (iXCapture == pTDepth->iXCapture) {
				++pTDepth->iXMoves;
				rlsp = pTDepth->NextMove;
			}
			else {
				pTDepth->iXMoves = 1;
				pTDepth->iXCapture = iXCapture;
				rlsp = pTDepth->FirstMove;
			}

			rlsp->bbField[BB_WHITEPIECE] = bbOpponent;
			rlsp->bbField[BB_BLACKPIECE] = (pbbField[BB_BLACKPIECE] ^ (pTDepth->bbFrom ^ bbPosition));
			rlsp->bbField[BB_KING] = (pbbField[BB_KING] & ~bbCaptured) ^ (pTDepth->bbFrom ^ bbPosition);

			if (pTDepth->iXCapture < 4 || pTDepth->iXMoves == 1 || !Duplicate_Move128(pTDepth, rlsp))
				pTDepth->NextMove = rlsp + 1;
			else
				--pTDepth->iXMoves;
		}

		if (!(bbEmpty & (bbPosition = DBL(bbPosition)))) {

			if (bbOpponent & bbPosition & DFR(bbEmpty))
				BlackKingNextCapture3_128(pTDepth, DBL(bbPosition), bbOpponent ^ bbPosition);

			break;
		}
	}
}

void CMoveGenerate128::WhitePieceMoveCount128(PTSTACK pTDepth)
{
	BITBOARD bbPosition, bbPosition0, *pbbField;

	pbbField = pTDepth->pbbField;

	BITBOARD bbWhitePiece = pbbField[BB_WHITEPIECE];
	BITBOARD bbKing = pbbField[BB_KING];
	BITBOARD bbWhiteKing = bbWhitePiece & bbKing;
	BITBOARD bbEmpty = pbbField[BB_EMPTY];

	//	WhiteMan Move
	BITBOARD bbWhiteManMove0 = DBR(bbEmpty) & (bbWhitePiece & ~bbKing);
	BITBOARD bbWhiteManMove1 = DBL(bbEmpty) & (bbWhitePiece & ~bbKing);

	int iXMoves = (int)POPCNT64(bbWhiteManMove0) + (int)POPCNT64(bbWhiteManMove1);

	//	WhiteKing Move
	while (bbWhiteKing) {

		bbPosition0 = bbWhiteKing & (~bbWhiteKing + 1);
		bbWhiteKing ^= bbPosition0;

		bbPosition = bbPosition0;
		while (bbEmpty & (bbPosition = DFL(bbPosition))) 
			++iXMoves;

		bbPosition = bbPosition0;
		while (bbEmpty & (bbPosition = DFR(bbPosition)))
			++iXMoves;

		bbPosition = bbPosition0;
		while (bbEmpty & (bbPosition = DBR(bbPosition)))
			++iXMoves;

		bbPosition = bbPosition0;
		while (bbEmpty & (bbPosition = DBL(bbPosition)))
			++iXMoves;
	}

	pTDepth->iXMoves = iXMoves;
}

void CMoveGenerate128::WhitePieceMove128(PTSTACK pTDepth)
{
	BITBOARD bbPosition, bbPosition0, bbToPosition, *pbbField;

	pbbField = pTDepth->pbbField;

	BITBOARD bbWhitePiece = pbbField[BB_WHITEPIECE];
	BITBOARD bbBlackPiece = pbbField[BB_BLACKPIECE];
	BITBOARD bbKing = pbbField[BB_KING];
	BITBOARD bbWhiteKing = bbWhitePiece & bbKing;
	BITBOARD bbEmpty = pbbField[BB_EMPTY];

	int iXMoves = 0;
	PLOOK rlsp = pTDepth->NextMove;

//	WhiteMan Move
	BITBOARD bbWhiteManMove0 = DBR(bbEmpty) & (bbWhitePiece & ~bbKing);
	BITBOARD bbWhiteManMove1 = DBL(bbEmpty) & (bbWhitePiece & ~bbKing);
	BITBOARD bbWhiteManMove = bbWhiteManMove0 | bbWhiteManMove1;

	while (bbWhiteManMove) {

		bbPosition = bbWhiteManMove & (~bbWhiteManMove + 1);
		bbWhiteManMove ^= bbPosition;

		if (bbWhiteManMove0 & bbPosition) {

			bbToPosition = DFL(bbPosition);

			rlsp->bbField[BB_WHITEPIECE] = (bbWhitePiece ^ (bbPosition ^ bbToPosition));
			rlsp->bbField[BB_BLACKPIECE] = bbBlackPiece;
			rlsp->bbField[BB_KING] = bbKing | (bbToPosition & DFLD_PROMOTION_WHITE);

			++iXMoves;
			++rlsp;
		}

		if (bbWhiteManMove1 & bbPosition) {

			bbToPosition = DFR(bbPosition);

			rlsp->bbField[BB_WHITEPIECE] = bbWhitePiece ^ (bbPosition ^ bbToPosition);
			rlsp->bbField[BB_BLACKPIECE] = bbBlackPiece;
			rlsp->bbField[BB_KING] = bbKing | (bbToPosition & DFLD_PROMOTION_WHITE);

			++iXMoves;
			++rlsp;
		}
	}

//	WhiteKing Move
	while (bbWhiteKing) {

		bbPosition0 = bbWhiteKing & (~bbWhiteKing + 1);
		bbWhiteKing ^= bbPosition0;

		bbPosition = bbPosition0;
		while (bbEmpty & (bbPosition = DFL(bbPosition))) {

			// Copy Position BitBoards
			rlsp->bbField[BB_WHITEPIECE] = (bbWhitePiece ^ (bbPosition0 ^ bbPosition));
			rlsp->bbField[BB_BLACKPIECE] = bbBlackPiece;
			rlsp->bbField[BB_KING] = bbKing ^ (bbPosition0 ^ bbPosition);

			++rlsp;
			++iXMoves;
		}

		bbPosition = bbPosition0;
		while (bbEmpty & (bbPosition = DFR(bbPosition))) {

			// Copy Position BitBoards
			rlsp->bbField[BB_WHITEPIECE] = (bbWhitePiece ^ (bbPosition0 ^ bbPosition));
			rlsp->bbField[BB_BLACKPIECE] = bbBlackPiece;
			rlsp->bbField[BB_KING] = bbKing ^ (bbPosition0 ^ bbPosition);

			++rlsp;
			++iXMoves;
		}

		bbPosition = bbPosition0;
		while (bbEmpty & (bbPosition = DBR(bbPosition))) {

			// Copy Position BitBoards
			rlsp->bbField[BB_WHITEPIECE] = (bbWhitePiece ^ (bbPosition0 ^ bbPosition));
			rlsp->bbField[BB_BLACKPIECE] = bbBlackPiece;
			rlsp->bbField[BB_KING] = bbKing ^ (bbPosition0 ^ bbPosition);

			++rlsp;
			++iXMoves;
		}

		bbPosition = bbPosition0;
		while (bbEmpty & (bbPosition = DBL(bbPosition))) {

			// Copy Position BitBoards
			rlsp->bbField[BB_WHITEPIECE] = (bbWhitePiece ^ (bbPosition0 ^ bbPosition));
			rlsp->bbField[BB_BLACKPIECE] = bbBlackPiece;
			rlsp->bbField[BB_KING] = bbKing ^ (bbPosition0 ^ bbPosition);

			++rlsp;
			++iXMoves;
		}
	}

	pTDepth->iXMoves = iXMoves;
	pTDepth->NextMove = rlsp;
}

void CMoveGenerate128::BlackPieceMoveCount128(PTSTACK pTDepth)
{
	BITBOARD bbPosition, bbPosition0, *pbbField;

	pbbField = pTDepth->pbbField;

	BITBOARD bbBlackPiece = pbbField[BB_BLACKPIECE];
	BITBOARD bbKing = pbbField[BB_KING];
	BITBOARD bbBlackKing = bbBlackPiece & bbKing;
	BITBOARD bbEmpty = pbbField[BB_EMPTY];

	//	BlackMan Move
	BITBOARD bbBlackManMove0 = DFL(bbEmpty) & (bbBlackPiece & ~bbKing);
	BITBOARD bbBlackManMove1 = DFR(bbEmpty) & (bbBlackPiece & ~bbKing);

	int iXMoves = (int)POPCNT64(bbBlackManMove0) + (int)POPCNT64(bbBlackManMove1);

	//	BlackKing Move
	while (bbBlackKing) {

		bbPosition0 = bbBlackKing & (~bbBlackKing + 1);
		bbBlackKing ^= bbPosition0;

		bbPosition = bbPosition0;
		while (bbEmpty & (bbPosition = DFL(bbPosition)))
			++iXMoves;

		bbPosition = bbPosition0;
		while (bbEmpty & (bbPosition = DFR(bbPosition)))
			++iXMoves;

		bbPosition = bbPosition0;
		while (bbEmpty & (bbPosition = DBR(bbPosition)))
			++iXMoves;

		bbPosition = bbPosition0;
		while (bbEmpty & (bbPosition = DBL(bbPosition)))
			++iXMoves;
	}

	pTDepth->iXMoves = iXMoves;
}

void CMoveGenerate128::BlackPieceMove128(PTSTACK pTDepth)
{
	BITBOARD bbPosition, bbPosition0, bbToPosition, *pbbField;

	pbbField = pTDepth->pbbField;

	BITBOARD bbWhitePiece = pbbField[BB_WHITEPIECE];
	BITBOARD bbBlackPiece = pbbField[BB_BLACKPIECE];
	BITBOARD bbKing = pbbField[BB_KING];
	BITBOARD bbBlackKing = bbBlackPiece & bbKing;
	BITBOARD bbEmpty = pbbField[BB_EMPTY];

	int iXMoves = 0;
	PLOOK rlsp = pTDepth->NextMove;

//	BlackMan Move
	BITBOARD bbBlackManMove0 = DFL(bbEmpty) & (bbBlackPiece & ~bbKing);
	BITBOARD bbBlackManMove1 = DFR(bbEmpty) & (bbBlackPiece & ~bbKing);
	BITBOARD bbBlackManMove = bbBlackManMove0 | bbBlackManMove1;

	while (bbBlackManMove) {

		bbPosition = bbBlackManMove & (~bbBlackManMove + 1);
		bbBlackManMove ^= bbPosition;

		if (bbBlackManMove0 & bbPosition) {

			bbToPosition = DBR(bbPosition);

			rlsp->bbField[BB_WHITEPIECE] = bbWhitePiece;
			rlsp->bbField[BB_BLACKPIECE] = (bbBlackPiece ^ (bbPosition ^ bbToPosition));
			rlsp->bbField[BB_KING] = bbKing | (bbToPosition & DFLD_PROMOTION_BLACK);

			++iXMoves;
			++rlsp;
		}

		if (bbBlackManMove1 & bbPosition) {

			bbToPosition = DBL(bbPosition);

			rlsp->bbField[BB_WHITEPIECE] = bbWhitePiece;
			rlsp->bbField[BB_BLACKPIECE] = (bbBlackPiece ^ (bbPosition ^ bbToPosition));
			rlsp->bbField[BB_KING] = bbKing | (bbToPosition & DFLD_PROMOTION_BLACK);

			++iXMoves;
			++rlsp;
		}
	}

//	BlackKing Move
	while (bbBlackKing) {

		bbPosition0 = bbBlackKing & (~bbBlackKing + 1);
		bbBlackKing ^= bbPosition0;

		bbPosition = bbPosition0;
		while (bbEmpty & (bbPosition = DFL(bbPosition))) {

			// Copy Position BitBoards
			rlsp->bbField[BB_WHITEPIECE] = bbWhitePiece;
			rlsp->bbField[BB_BLACKPIECE] = (bbBlackPiece ^ (bbPosition0 ^ bbPosition));
			rlsp->bbField[BB_KING] = bbKing ^ (bbPosition0 ^ bbPosition);

			++rlsp;
			++iXMoves;
		}

		bbPosition = bbPosition0;
		while (bbEmpty & (bbPosition = DFR(bbPosition))) {

			// Copy Position BitBoards
			rlsp->bbField[BB_WHITEPIECE] = bbWhitePiece;
			rlsp->bbField[BB_BLACKPIECE] = (bbBlackPiece ^ (bbPosition0 ^ bbPosition));
			rlsp->bbField[BB_KING] = bbKing ^ (bbPosition0 ^ bbPosition);

			++rlsp;
			++iXMoves;
		}

		bbPosition = bbPosition0;
		while (bbEmpty & (bbPosition = DBR(bbPosition))) {
			// Copy Position BitBoards

			rlsp->bbField[BB_WHITEPIECE] = bbWhitePiece;
			rlsp->bbField[BB_BLACKPIECE] = (bbBlackPiece ^ (bbPosition0 ^ bbPosition));
			rlsp->bbField[BB_KING] = bbKing ^ (bbPosition0 ^ bbPosition);

			++rlsp;
			++iXMoves;
		}

		bbPosition = bbPosition0;
		while (bbEmpty & (bbPosition = DBL(bbPosition))) {

			// Copy Position BitBoards
			rlsp->bbField[BB_WHITEPIECE] = bbWhitePiece;
			rlsp->bbField[BB_BLACKPIECE] = (bbBlackPiece ^ (bbPosition0 ^ bbPosition));
			rlsp->bbField[BB_KING] = bbKing ^ (bbPosition0 ^ bbPosition);

			++rlsp;
			++iXMoves;
		}
	}

	pTDepth->iXMoves = iXMoves,
	pTDepth->NextMove = rlsp;
}

int CMoveGenerate128::MoveGenerateC128(PTSTACK pTDepth, bool bTurn)
{
	pTDepth->iXMoves = 0;															// Initialisatie 
	pTDepth->iXCapture = 0;
	pTDepth->NextMove = pTDepth->FirstMove;

	if (bTurn == true) {															// White to move 

		if ((QWhiteManCapture128(pTDepth) + QWhiteKingCapture128(pTDepth)) == 0)
			WhitePieceMoveCount128(pTDepth);										// White Piece Move

	}
	else {
		// Black to move  
		if ((QBlackManCapture128(pTDepth) + QBlackKingCapture128(pTDepth)) == 0)
			BlackPieceMoveCount128(pTDepth);										// Black Piece Move
	}

	(pTDepth + 1)->FirstMove = pTDepth->NextMove;

	return (pTDepth->iXMoves);
}

int CMoveGenerate128::MoveGenerate128(PTSTACK pTDepth, bool bTurn)
{
	pTDepth->iXMoves = 0;															// Initialisatie
	pTDepth->iXCapture = 0;
	pTDepth->NextMove = pTDepth->FirstMove;

	if (bTurn == true) {															// White to move 

		if ((QWhiteManCapture128(pTDepth) + QWhiteKingCapture128(pTDepth)) == 0)
			WhitePieceMove128(pTDepth);												// White Piece Move

	} else {
		// Black to move  
		if ((QBlackManCapture128(pTDepth) + QBlackKingCapture128(pTDepth)) == 0)
			BlackPieceMove128(pTDepth);												// Black Piece Move
	}

	(pTDepth + 1)->FirstMove = pTDepth->NextMove;

	return (pTDepth->iXMoves);
}

